let eval_hoareTriple e =
	let eval_duration t =
		match t with
			| Parser.ValueTime(vt) -> "ValueTime(" ^ string_of_float(float_of_string(vt)) ^ ")"
			| Parser.VarTime(vt) -> "VarTime(\"" ^ vt ^ "\")"
		in 
	let rec eval_symbol_slide s =
		match s with
			| '+' -> "PlusSlide"
			| '-' -> "MinusSlide"
			| _ -> raise(Invalid_argument "eval_symbol : Should have the '+', '-' symbol for Slide or none")
		in
	let eval_comp c =
		match c with
			| "<="|"=<" -> "LE"
			| "<" -> "LT"
			| ">="|"=>" -> "GE"
			| ">" -> "GT"
			| "=" -> "Eq"
			| "!=" -> "NEq"
			| _ -> raise(Invalid_argument "eval_comp: Need a comparator for the Celerities of assertions")
		in
	let rec eval_resources w = 
		match w with
			| Parser.Multiplex(m) -> m
			| Parser.SetMultiplexes(m,r) -> m ^ "," ^ eval_resources(r)
		in
	let eval_symbol_hterm s =
		match s with
			| '+' -> "Plus"
			| '-' -> "Minus"
			| '*' -> "Times"
			| '/' -> "Divide"
			| _ -> raise(Invalid_argument "eval_symbol_hterm : Should have +, -,* or / symbol in relation between Hybrid terms")
		in
	let rec eval_hybridTerm ht =
		match ht with
			| Parser.PiEntrance(p) -> "PiEntrance(" ^ p ^ ",0)"
			| Parser.PiExit(p) -> "PiExit(" ^ p ^ ",0)"
			| Parser.Celerity(v,w,n) -> "Celerity(" ^ v ^ "," ^ eval_resources(w) ^ "," ^ n ^ ")"
			| Parser.HTermRelation(ha1,connective,ha2) -> "HTermRelation(" ^ eval_symbol_hterm(connective) ^ "," ^ eval_hybridTerm(ha1) ^ "," ^ eval_hybridTerm(ha2) ^ ")"
			| Parser.HConstant(cst) -> "HConstant(" ^ string_of_float(float_of_string(cst)) ^ ")"
			| Parser.HTime(t) -> "HTime(" ^ eval_duration(t) ^ ")"
		in
	let rec eval_assertion ea =
		match ea with
			| Parser.ASlide(v)-> "ASlide(EqSlide,\"" ^ v ^ "\")"
			| Parser.ASlideSymbol(s,v)-> "ASlide(" ^ eval_symbol_slide(s) ^ ",\"" ^ v ^ "\")"
			| Parser.ACelerity(v,comp,value)-> "ACelerity(" ^ eval_comp(comp) ^ ",\"" ^ v ^ "\"," ^ eval_hybridTerm(value) ^ ")"
			| Parser.ABoolean(a)-> "ABoolean(" ^ a ^ ")"
			| Parser.AOpAnd(a1,a2)-> "AOp(And," ^ eval_assertion(a1) ^ "," ^ eval_assertion(a2) ^ ")"
			| Parser.AOpOr(a1,a2)-> "AOp(Or," ^ eval_assertion(a1) ^ "," ^ eval_assertion(a2) ^ ")"
			| Parser.ANeg(a)-> "ANeg(Neg," ^ eval_assertion(a) ^ ")"
			| Parser.APar(a)-> eval_assertion(a)
		in
	let eval_symbol s =
		match s with
			| '+' -> "PlusSymbol"
			| '-' -> "MinusSymbol"
			| _ -> raise(Invalid_argument "eval_symbol : Should have + or - symbol for the discrete path atom")
		in
	let eval_dpa d =
		match d with
			| Parser.DpaValue(v,s) -> "(\"" ^ v ^ "\"," ^ eval_symbol(s) ^ ")"
		in
	let eval_elementaryPath ep =
		match ep with
			| Parser.Tuple(d,at,dpa) -> "Tuple(" ^ eval_duration(d) ^ "," ^ eval_assertion(at) ^ "," ^ eval_dpa(dpa) ^ ")"
		in
	let eval_symbol_dterm s =
		match s with
			| '+' -> "DPlus"
			| '-' -> "DMinus"
			| _ -> raise(Invalid_argument "eval_symbol_dterm : Should have + or - symbol in relation between Discrete terms")
		in
	let eval_discreteTerm d =
		match d with
			| Parser.Eta(v) -> "Eta(\"" ^ v ^ "\")"
			| Parser.DConstant(v) -> "DConstant(" ^ v ^ ")"
		in
	let rec eval_discreteAtom d =
		match d with
			| Parser.DTermUn(dt) -> eval_discreteTerm(dt)
			| Parser.DTermRelation(dt1,connective,dt2) -> "DTermRelation(" ^ eval_symbol_dterm(connective) ^ "," ^ eval_discreteAtom(dt1) ^ "," ^ eval_discreteAtom(dt2) ^ ")" 
		in
	let rec eval_discreteCondition d =
		match d with
			| Parser.DBoolean(c1) -> "DBoolean(" ^ c1 ^ ")"
			| Parser.DAtom(d1,comp,d2) -> "DAtom(" ^ eval_comp(comp) ^ "," ^  eval_discreteAtom(d1) ^ "," ^  eval_discreteAtom(d2) ^ ")"
			| Parser.DOpAnd(d1,d2) -> "DOp(And," ^ eval_discreteCondition(d1) ^ "," ^ eval_discreteCondition(d2) ^ ")"
			| Parser.DOpOr(d1,d2) -> "DOp(Or," ^ eval_discreteCondition(d1) ^ "," ^ eval_discreteCondition(d2) ^ ")"
			| Parser.DNeg(d1) -> "DNeg(Neg," ^ eval_discreteCondition(d1) ^ ")"
			| Parser.DPar(d1) -> eval_discreteCondition(d1)
		in
	let rec eval_hybridCondition c =
		match c with
			| Parser.HBoolean(h1) -> "HBoolean(" ^ h1 ^ ")"
			| Parser.HAtom(h1,comp,h2) -> "HAtom(" ^ eval_comp(comp) ^ "," ^ eval_hybridTerm(h1) ^ "," ^ eval_hybridTerm(h2) ^ ")"
			| Parser.HDAtom(hd1,comp,hd2) -> "HDAtom(" ^ eval_comp(comp) ^ "," ^ eval_discreteAtom(hd1) ^ "," ^ eval_discreteAtom(hd2) ^ ")"
			| Parser.HOpAnd(h1,h2) -> "HOp(And," ^ eval_hybridCondition(h1) ^ "," ^ eval_hybridCondition(h2) ^ ")"
			| Parser.HOpOr(h1,h2) -> "HOp(Or," ^ eval_hybridCondition(h1) ^ "," ^ eval_hybridCondition(h2) ^ ")"
			| Parser.HNeg(h1) -> "HNeg(" ^ eval_hybridCondition(h1) ^ ")"
			| Parser.HPar(h1) -> "(" ^ eval_hybridCondition(h1) ^ ")"
		in
	let rec eval_path p =
		match p with
			| Parser.EPath(ep) -> eval_elementaryPath(ep)
			| Parser.Seq(ep,p) -> "Seq(" ^ eval_elementaryPath(ep) ^ "," ^ (eval_path p) ^ ")" 
		in
	let eval_post p =
		match p with
			| Parser.PostCondition(dc,hc) -> "d0 = " ^ eval_discreteCondition(dc) ^ " and h0 = " ^ eval_hybridCondition(hc)
		in
	let rec eval_multformula m =
		match m with
			| Parser.MAtom(v,comp,i) -> (match comp with
						| "=" -> "MPropBin(And,MAtom(\"" ^ v ^ "\"," ^ i ^ "),MPropUn(Neg,MAtom(\"" ^ v ^ "\"," ^ i ^ "+1)))"
						| "!=" -> "MPropUn(Neg,MPropBin(And,MAtom(\"" ^ v ^ "\"," ^ i ^ "),MPropUn(Neg,MAtom(\"" ^ v ^ "\"," ^ i ^ "+1))))"
						| ">" -> "MAtom(\"" ^ v ^"\"," ^ i ^ "+1)"
						| ">="|"=>" -> "MAtom(\"" ^ v ^ "\"," ^ i ^ ")"
						| "<" -> "MPropUn(Neg,MAtom(\"" ^ v ^ "\"," ^ i ^ "))"
						| "<="|"=<" -> "MPropUn(Neg,MAtom(\"" ^ v ^ "\"," ^ i ^ "+1))"
						| _ -> raise(Invalid_argument "eval_Mcomp : not a comparator !"))
			| Parser.MPropNeg(m1) -> "MPropUn(Neg," ^ eval_multformula(m1) ^ ")" 
			| Parser.MPropAnd(m1,m2) -> "MPropBin(And," ^ eval_multformula(m1) ^ "," ^ eval_multformula(m2) ^ ")"
			| Parser.MPropOr(m1,m2) -> "MPropBin(Or," ^ eval_multformula(m1) ^ "," ^ eval_multformula(m2) ^ ")"
			| Parser.MPropPar(m1) -> eval_multformula(m1)
		in
	let eval_formula f =
		match f with
			| Parser.MFormula(f1) -> eval_multformula(f1)
		in
	let rec eval_varlist v =
		match v with
			| Parser.MVar(v1) -> v1
			| Parser.MVarList(v1,v2) -> v1 ^ "," ^ eval_varlist(v2)
		in
	let eval_target m t =
		match t with
			| Parser.MTarget(t1) -> "Target(" ^ m ^ ",[" ^ eval_varlist(t1) ^ "])"
		in
	let eval_predecessor p =
		match p with
			| Parser.VarDefinition(v,n) -> "(\"" ^ v ^ "\", (" ^ n ^",[]))"
			| Parser.MultDefinition(m,f,t) -> "(\"" ^ m ^"\"," ^ eval_formula(f) ^ "), " ^ (eval_target m t)
		in
	let rec eval_influenceGraph s =
		match s with
			| Parser.PredecessorDef(p) -> eval_predecessor(p)
			| Parser.SeveralVarDef(p,s1) -> eval_predecessor(p) ^ ";" ^ eval_influenceGraph(s1)
		in
	let identify_path e =
		match e with
			| Parser.HoareTriple(path,post) -> "let prog0 = " ^ eval_path(path) ^ ";;\nlet post0 = (d0,h0) with\n" ^ eval_post(post) ^ ";;\n"
			| Parser.InfluenceGraph(s) -> "let vars+mults = [" ^ eval_influenceGraph(s) ^ "];;\n"
	in identify_path e
;;