open Types;;
open Extraction_data;;
open Standard_functions;;
open Data;;
open Valuation;;
open Wp_computation;;
open Dnf_constraints;;
open Output_files;;
open Display;;
open Printf;;


(*******************************)
(*** Circadian cycle example ***)
(*******************************)

(* Toy example for tests *)
(* let vars =
  [("pc",(1,["m2"])) ;
  ("g",(1,["m1" ; "m3"])) ;
  ("L",(1,["m5"])) ;
  ("X",(1,["m4"]))];;

let mults =
  [("m1", MPropUn(Neg, MAtom("pc", 1))) ;
  ("m2", MAtom("g", 1)) ;
  ("m3", MAtom("L", 1)) ;
  ("m4", MAtom("L", 1)) ;
  ("m5", MPropUn(Neg, MAtom("X", 1)))
  ];; *)


(*******************************)
(*** Hoare triple definition ***)
(*******************************)


let _ =
  (* signed_celerity is a function adding the constraint : C_{v,omega,n} * C_{v,omega,(n+1)} => 0  
and C_{v,omega,n} = 0 => ∀ k in [0;n-1],C_{v,omega,n} > 0  ∧ ∀ k in [n+1;b_v],C_{v,omega,n} < 0*)
  h0 := HOp(And, signed_celerity varlist, !h0) in
  let post0 = (d0, !h0) in 
  let pos_list = pi_string_list prog0 in
  (** Final result: call the functions above on the example **)
  let wp = computeWP prog0 post0 in
  let simpl_wp = (simpl_valuation << filter_out_true) wp in
  let list_of_cel = celerities_of_wp prog0 post0 in
  print_endline (print_cel list_of_cel);
  (* print_endline (print_lpost (discreteWPSP prog0 (discretePart post0))); *)
  (* print_lsuper_valuation simpl_wp ; *)
  let s = separate_disj simpl_wp list_of_cel in
  (* let dnf_result = simplify_cstr_pi (s) in
  print_endline (print_hConjl (dnf_result));
  print_endline (print_hDisjl (dnf_result)); *)
  let dnf_result = simplify_cstr_touch_delay (simplify_cstr_cel (simplify_cstr_pi (s))) in
  (* print_endline (print_hDisjl_with_simplify_cstr_touch_delay (dnf_result));
  print_endline (print_select_hdisj (select_hdisj dnf_result));
  print_endline (print_replace_HOp_in_hdisjl (replace_HOp_in_hdisjl (select_hdisj dnf_result))); *)
  (* print_endline (print_comb_elt_in_disj (comb_elt_in_disj (replace_HOp_in_hdisjl (select_hdisj dnf_result)))); *)
  (* print_endline (print_hConjl_simplified (simplify_hconj (replace_HOp_in_hconl (get_hconj dnf_result)))); *)
  let final_disj = comb_elt_in_disj (replace_HOp_in_hdisjl (select_hdisj dnf_result)) in
  let final_hconj = simplify_hconj (replace_HOp_in_hconl (get_hconj dnf_result)) in
  if (isCyclic) 
  then (create_file Sys.argv.(1) (get_lval dnf_result) (final_hconj) final_disj pos_list cel_list (cyclic_behaviour prog0)) 
  else (create_file Sys.argv.(1) (get_lval dnf_result) (final_hconj) final_disj pos_list cel_list [""])
;;