(********************************)
(*** Creation of output files ***)
(********************************)

open Types;;
open Display;;

(* get_lval function select lvaluation list to display into the output files *)
let get_lval sval =
  let rec select_lval s =
    match s with
      | (lval, (_,_)) -> lval
    in
  let rec select_element s l =
    match s with
      | [] -> raise(Invalid_argument "select_element: lvaluation empty")
      | h::[] -> [select_lval h] @ l
      | h::t -> select_element t ([select_lval h] @ l)
    in
    select_element sval [];;


(* This function permits to create output files. Each file contains each formula in
conjunction. All files are separated by the disjunction connective. *)
open Printf;;
let create_file file lval_list hconj_list hdisj_list pl cl cycle =
  let rec write_val oc v =
    match v with
      | [] -> raise(Invalid_argument "write_val: valuation list empty")
      | h :: [] -> (match h with
              | (ht, fl) -> (match ht with
                        | PiEntrance(v,i) -> fprintf oc "%s;\n" (string_of_couple string_of_hybridTerm_solver string_of_float "=" "" "" (ht,fl))
                        | PiExit(v,i) -> fprintf oc "%s;\n" (string_of_couple string_of_hybridTerm_solver string_of_float "=" "" "" (ht,fl))
                        | Celerity(v,r,i) -> fprintf oc "%s;\n" (string_of_couple string_of_hybridTerm_solver string_of_float "=" "" "" (ht,fl))
                        | _ -> raise(Invalid_argument "write_val: The variables are only pi_entrance, pi_exit and celerities")))
      | h :: t -> (match h with
              | (ht, fl) -> (match ht with
                        | PiEntrance(v,i) -> fprintf oc "%s;\n" (string_of_couple string_of_hybridTerm_solver string_of_float "=" "" "" (ht,fl))
                        | PiExit(v,i) -> fprintf oc "%s;\n" (string_of_couple string_of_hybridTerm_solver string_of_float "=" "" "" (ht,fl))
                        | Celerity(v,r,i) -> fprintf oc "%s;\n" (string_of_couple string_of_hybridTerm_solver string_of_float "=" "" "" (ht,fl))
                        | _ -> raise(Invalid_argument "write_val: The variables are only pi_entrance, pi_exit and celerities"))); write_val oc t
    in
  let rec write_hconj oc hc =
    match hc with
      | [] -> raise(Invalid_argument "write_hconj: hybridConjunction list empty")
      | h :: [] -> fprintf oc "%s;\n" (string_of_hybridCondition_solver h)
      | h :: t -> fprintf oc "%s;\n" (string_of_hybridCondition_solver h); write_hconj oc t
    in
  let rec write_list oc pl =
    match pl with
      | [] -> raise(Invalid_argument "write_pi: pi_entrance and pi_exit list empty")
      | h :: [] -> fprintf oc "%s" h
      | h :: t -> fprintf oc "%s" h; write_list oc t
    in
  let list_is_empty l =
    match l with
      | [] -> true
      | h :: t -> false
    in
  let cr pl cl file h v hc n m cycle =
    let oc = open_out (file ^ "_" ^ (string_of_int n) ^ "_" ^ (string_of_int m) ^ ".bch") in
      fprintf oc "Variables\n";
      write_list oc pl;
      write_list oc cl;
      fprintf oc "\nConstraints\n";
      write_val oc v;
      write_hconj oc hc;
      write_list oc cycle;
      if (list_is_empty h)
      then (fprintf oc "end";)
      else fprintf oc "%s;\nend" (string_of_list string_of_hybridCondition_solver ";\n" h);
      close_out oc
    in
    (* Look for each elt in hdisj list and call create function to create file on each
    lval and hconj elt *)
  let rec select_each_elt f pl cl hd v hc n m cycle =
    match hd with
      | [] -> cr pl cl f [] v hc n m cycle
      | h::[] -> cr pl cl f h v hc n m cycle
      | h::t -> cr pl cl f h v hc n m cycle; select_each_elt f pl cl t v hc n (m+1) cycle
    in
  let rec select_each_elt_hdisj_list f pl cl hd v hc n cycle =
    match hd with
      | [] -> raise(Invalid_argument "create : Error")
      | h::[] -> select_each_elt f pl cl h v hc n 1 cycle
      | h::t ->  select_each_elt f pl cl h v hc n 1 cycle; select_each_elt_hdisj_list f pl cl t v hc n cycle
    in
    (* Return a tuple (first elt of hconj list, hconj list withour first elt) *)
  let select_elt_hconj_list hconj =
    match hconj with
      | [] -> raise(Invalid_argument "select_elt_hconj_list: Error")
      | h::[] -> (h,[])
      | h::t -> (h,t)
    in
    (* Look for each elt in lval and in hconj lists *)
  let rec select_element file lvall hconjl hdisjl pl cl n cycle =
    match lvall with
      | [] -> raise(Invalid_argument "select_element: Error")
      | h1::[] -> select_each_elt_hdisj_list file pl cl hdisjl h1 (fst(select_elt_hconj_list hconjl)) n cycle
      | h1::t1 -> select_each_elt_hdisj_list file pl cl hdisjl h1 (fst(select_elt_hconj_list hconjl)) n cycle; select_element file t1 (snd(select_elt_hconj_list hconjl)) hdisjl pl cl (n+1) cycle
    in select_element file lval_list hconj_list hdisj_list pl cl 1 cycle ;;