#!/bin/sh
echo "
*************\n
*** Start ***\n
*************\n" && \

# select path if exists 
filename=$(basename $1) && \
ocamllex lexer.mll && \
ocamlyacc parser.mly && \
echo "type durationTerm =
	ValueTime of string
	| VarTime of string

type dpaTerm =
	DpaValue of string * char

type discreteTerm =
	Eta of string
	| DConstant of string

type discreteAtom = 
	DTermUn of discreteTerm
	| DTermRelation of discreteAtom * char * discreteAtom

type discreteCondition =
	DBoolean of string
	| DAtom of discreteAtom * string * discreteAtom
	| DOpAnd of discreteCondition * discreteCondition
	| DOpOr of discreteCondition * discreteCondition
	| DNeg of discreteCondition
	| DPar of discreteCondition

type resource =
	Multiplex of string
	| SetMultiplexes of string * resource

type hybridTerm =
	PiEntrance of string
	| PiExit of string
	| Celerity of string * resource * string
	| CelerityNR of string * string
	| HTermRelation of hybridTerm * char * hybridTerm
	| HConstant of string
	| HTime of durationTerm

type hybridCondition =
	HBoolean of string
	| HAtom of hybridTerm * string * hybridTerm
	| HDAtom of discreteAtom * string * discreteAtom
	| HOpAnd of hybridCondition * hybridCondition
	| HOpOr of hybridCondition * hybridCondition
	| HNeg of hybridCondition
	| HPar of hybridCondition

type assertionTerm =
	ASlide of string
	| ASlideSymbol of char * string
	| ACelerity of string * string * hybridTerm
	| ABoolean of string
	| AOpAnd of assertionTerm * assertionTerm
	| AOpOr of assertionTerm * assertionTerm
	| ANeg of assertionTerm
	| APar of assertionTerm

type elementaryPath =
	Tuple of durationTerm * assertionTerm * dpaTerm

type path = 
	EPath of elementaryPath
	| Seq of elementaryPath * path

type postcondition =
	PostCondition of discreteCondition * hybridCondition

type multformula =
	MAtom of string * string * string
	| MPropNeg of multformula
	| MPropAnd of multformula * multformula
	| MPropOr of multformula * multformula
	| MPropPar of multformula

type varlist =
	MVar of string
	| MVarList of string * varlist

type target =
	MTarget of varlist

type formula =
	MFormula of multformula

type predecessor = 
	VarDefinition of string * string
	| MultDefinition of string * formula * target

type influenceGraph =
	PredecessorDef of predecessor
	| SeveralVarDef of predecessor * influenceGraph

type fileLine = 
	HoareTriple of path * postcondition
	| InfluenceGraph of influenceGraph
	| IsCyclic of bool\n" | cat - parser.mli > temp && mv temp parser.mli && \
ocamlc -c parser.mli && \
ocamlc -c lexer.ml && \
ocamlc -c parser.ml && \
ocamlc -c types.ml && \
ocamlc -c display.ml && \
ocamlc -c input.ml && \
ocamlc -o input lexer.cmo parser.cmo types.cmo display.cmo input.cmo  && \
./input $1 && \
ocamlc -c types.ml && \
ocamlc -c display.ml && \
ocamlc -c data.ml && \
ocamlc -c extraction_data.ml && \
ocamlc -c standard_functions.ml && \
ocamlc -c dnf_discrete_condition.ml && \
ocamlc -c set_states.ml && \
ocamlc -c strongest_postcondition.ml && \
ocamlc -c wp_properties.ml && \
ocamlc -c simplification_hybrid_condition.ml && \
ocamlc -c valuation.ml && \
ocamlc -c wp_computation.ml && \
ocamlc -c dnf_constraints.ml && \
ocamlc -c output_files.ml && \
ocamlc -c main.ml && \
ocamlc -o main types.cmo display.cmo data.cmo extraction_data.cmo standard_functions.cmo dnf_discrete_condition.cmo set_states.cmo strongest_postcondition.cmo wp_properties.cmo simplification_hybrid_condition.cmo valuation.cmo wp_computation.cmo dnf_constraints.cmo output_files.cmo main.cmo && \
if [ $# -eq 2 ]
	then
		newDir=$(dirname $2)
		if [ -d "$newDir" ]
			then
				./main $2
			else
				mkdir $newDir
				./main $2
		fi
	else
		./main $filename
fi
rm -f *.cmi *.cmo *.mli lexer.ml parser.ml data.ml input main && \
echo "\n
************\n
*** Done ***\n
************"